""" starcli.__main__ """

from argparse import Namespace

# Internal application imports
from .layouts import list_layout, table_layout
from .search import search
from .parser import args


def cli(args={}):
    if args == {}:  # if args is not provided
        args = Namespace(
            lang=None,
            stars=">=50",
            date=None,
            debug=False,
            layout=None,
            limit_results=7,
            order="desc",
        )

    if args.debug:
        print(f"DEBUG: cli(): received args is {args}")

    if not args.stars:  # if args.stars is not present
        args.stars = ">=50"
    tmp_repos = search(args.lang, args.date, args.stars, args.debug, args.order)
    if not tmp_repos:  # if search() returned None
        return

    repos = []
    for i in range(args.limit_results):
        repos.append(tmp_repos[i])

    if args.layout == "table":
        table_layout(repos)
        return
    list_layout(repos)


if __name__ == "__main__":
    cli(args)



from argparse import ArgumentParser

parser = ArgumentParser(description="Browse trending repos on GitHub by stars")
parser.add_argument(
    "-l", "--lang", type=str, help="Language filter eg:python", required=False
)
parser.add_argument(
    "-d",
    "--date",
    type=str,
    help="Specify repo creation date in ISO8601 format YYYY-MM-DD",
    required=False,
)
parser.add_argument(
    "-L",
    "--layout",
    type=str,
    choices=("list", "table"),
    default="list",
    help="The output format (list or table), default is list",
)
parser.add_argument(
    "-s",
    "--stars",
    type=str,
    required=False,
    default=">=50",
    help="Range of stars required, default is '>=50'",
)
parser.add_argument(
    "-r",
    "--limit-results",
    type=int,
    default=7,
    help="Limit the number of results shown. Default: 7",
)
parser.add_argument(
    "-o",
    "--order",
    type=str,
    choices=("desc", "asc"),
    default="desc",
    help="Specify the order of repos by stars that is shown, 'desc' or 'asc', default: desc",
)
parser.add_argument("--debug", action="store_true", help="Turn on debugging mode")
args = parser.parse_args()



""" starcli.terminal_size """


def terminal_size():
    """ Fetches the terminal size and returns it """
    import fcntl, termios, struct

    th, tw, hp, wp = struct.unpack(
        "HHHH", fcntl.ioctl(0, termios.TIOCGWINSZ, struct.pack("HHHH", 0, 0, 0, 0))
    )
    return tw, th



""" starcli.search """

# Standard library imports
from datetime import datetime, timedelta

# Third party imports
import requests


API_URL = "https://api.github.com/search/repositories"


def search(language=None, date=None, stars=">=50", debug=False, order="desc"):
    """ Returns repositories based on the language.
        repositories are sorted by stars
    """
    date_format = "%Y-%m-%d"  # date format in iso format
    if debug:
        print("DEBUG: search: date param:", date)
        print("DEBUG: search: order param: ", order)

    if not date:
        start_date = (datetime.utcnow() + timedelta(days=-100)).strftime(date_format)
        end_date = (datetime.utcnow() + timedelta(days=1)).strftime(date_format)
    else:
        try:
            tmp_date = datetime.strptime(date, date_format)
        except ValueError:
            print("invalid date format: " + date + " must be yyyy-mm-dd")
            return
        end_date = (tmp_date + timedelta(days=1)).strftime(date_format)
        start_date = tmp_date.strftime(date_format)

    if debug:  # print start_date and end_date if debugging
        print("DEBUG: search: start date:", start_date)
        print("DEBUG: search: end_date:", end_date)

    query = f"stars:{stars}+created:{start_date}..{end_date}"
    query += f"+language:{language}" if language else ""
    url = f"{API_URL}?q={query}&sort=stars&order={order}"
    if debug:
        print("DEBUG: search: url:", url)  # print the url when debugging

    repositories = requests.get(url).json()

    return repositories["items"]



""" starcli.layouts """

# Standard library imports
import textwrap

# Third party imports
from rich.console import Console
from rich.table import Table

from .terminal_size import terminal_size


def list_layout(repos):
    """ Displays repositories in list layout using rich """

    console = Console()  # initialise rich
    separator = "+==============================================================+"
    term_width, _ = terminal_size()
    side_width = int((term_width - len(separator)) / 2)

    console.print(separator, justify="center", end="\n\n")
    for repo in repos:
        console.print(
            " " * side_width,
            f"[link={repo['html_url']}]{repo['full_name']}[/link]",
            style="yellow",
            end="  ",
        )

        stats = (
            str(repo["stargazers_count"])
            + ":star:, "
            + str(repo["forks_count"])
            + ":fork_and_knife:, "
            + str(repo["watchers_count"])
            + ":eyes:"
        )

        if len(repo["full_name"] + stats) > len(separator + "   "):
            print()
            console.print(
                " " * ((side_width) + (len(separator) - len(stats))), stats, end="\n\n"
            )
        else:
            console.print(stats, end="\n\n")

        console.print(" " * side_width, repo["language"], style="bold cyan", end="\n\n")
        console.print(
            " " * side_width,
            ("\n" + " " * side_width).join(
                textwrap.wrap(f"{repo['description']}", len(separator))
            ),
            style="green",
            end="\n\n",
        )
        console.print(separator, justify="center", end="\n\n")


def table_layout(repos):
    """ Displays repositories in a table format using rich """
    table = Table()

    table.add_column("Name", style="bold cyan", no_wrap=True, width=45)
    table.add_column("Language", style="green", no_wrap=True, width=23)
    table.add_column("Description", style="blue", no_wrap=False, width=160)
    table.add_column("Stats", style="magenta", no_wrap=True, width=45)

    for repo in repos:
        stats = (
            str(repo["stargazers_count"])
            + ":star:, "
            + str(repo["forks_count"])
            + ":fork_and_knife:, "
            + str(repo["watchers_count"])
            + ":eyes:"
        )

        if not repo["language"]:  # if language is not provided
            repo["language"] = "None"  # make it a string
        if not repo["description"]:
            repo["description"]: "None"

        table.add_row(
            repo["name"] + "\n\n",
            repo["language"] + "\n\n",  # so that it can work here
            repo["description"],
            stats,
        )

    console = Console()
    console.print(table)



""" setup """

import io

from setuptools import setup

with io.open("README.md", "rt", encoding="utf8") as f:
    long_desc = f.read()

VERSION = "2.6.2"

# This call to setup() does all the work
setup(
    name="starcli",
    version=VERSION,
    description="Browse popular repos on github by star trends from your command line!",
    long_description=long_desc,
    long_description_content_type="text/markdown",
    python_requires=">=3.6",
    url="https://github.com/hedythedev/starcli",
    author="Hedy Li",
    author_email="hedyhyry@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    packages=["starcli"],
    include_package_data=True,
    install_requires=["requests>=2.22.0", "rich>=2.1.0",],
    entry_points={"console_scripts": ["starcli=starcli.__main__:cli",]},
)
